@@ -13,13 +13,16 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+#include"config.h"
+
+#include<cstdarg>
 #include<cstdio>
 #include<cstring>
-#include<cstdarg>
 #include<fstream>
 #include<functional>
 #include<iomanip>
 #include<iostream>
+#include<libgen.h>
 #include<map>
 #include<sstream>
 #include<stdexcept>
@@ -28,8 +31,12 @@
 #include<unistd.h>
 #include<vector>
 
-#include"openssl/rsa.h"
+#include"openssl/err.h"
+#include"openssl/ossl_typ.h"
 #include"openssl/rand.h"
+#include"openssl/rsa.h"
+#include"openssl/x509.h"
+
 #include"tss/tspi.h"
 #include"trousers/trousers.h"
 
@@ -37,6 +44,52 @@
 #include"tspiwrap.h"
 #include"internal.h"
 
+namespace {
+
+// In OpenSSL 1.1 these functions replaced direct access to internals
+// of the `RSA` struct.
+// This is local implementations so that we can still build under
+// OpenSSL 1.0.
+#ifndef HAVE_RSA_SET0_KEY
+int
+RSA_set0_key(RSA *rsa, BIGNUM *n, BIGNUM *e, BIGNUM *d)
+{
+  rsa->n = n;
+  rsa->e = e;
+  rsa->d = d;
+  return 0;
+}
+#endif
+#ifndef HAVE_RSA_GET0_KEY
+void
+RSA_get0_key(RSA *rsa, const BIGNUM **n, const BIGNUM **e, const BIGNUM **d)
+{
+  if (n) {
+    *n = rsa->n;
+  }
+  if (e) {
+    *e = rsa->e;
+  }
+  if (d) {
+    *d = rsa->d;
+  }
+}
+#endif
+#ifndef HAVE_RSA_GET0_FACTORS
+void
+RSA_get0_factors(const RSA *rsa, const BIGNUM **p, const BIGNUM **q)
+{
+  if (p != NULL) {
+    *p = rsa->p;
+  }
+  if (q != NULL) {
+    *q = rsa->q;
+  }
+}
+#endif
+}
+
+
 std::ostream&
 operator<<(std::ostream& o, const struct stpm::Key& key)
 {
@@ -57,15 +110,60 @@ operator<<(std::ostream& o, const struct stpm::SoftwareKey& key)
 
 BEGIN_NAMESPACE(stpm);
 const std::string random_device = "/dev/urandom";
-const int num_random_bytes = 32; // 256 bits.
+const int num_random_bytes = 10240; // 10*8192 bits.
 const char* env_log_stderr = "SIMPLE_TPM_PK11_LOG_STDERR";
 const TSS_UUID srk_uuid = TSS_UUID_SRK;
 
+BEGIN_NAMESPACE();
+template<typename T, T*(*New)(), void(*Free)(T*)>
+class AutoFree {
+ public:
+  AutoFree(): resource_(New()) {}
+  AutoFree(T* r): resource_(r) {}
+  AutoFree(const AutoFree&) = delete;
+  AutoFree(const AutoFree&&) = delete;
+  AutoFree& operator=(const AutoFree&) = delete;
+  AutoFree& operator=(const AutoFree&&) = delete;
+  ~AutoFree()
+  {
+    if (!resource_) {
+      return;
+    }
+    try {
+      Free(resource_);
+      resource_ = nullptr;
+    } catch (const std::exception& e) {
+      std::clog << "Exception thrown in free() code.\n";
+      throw;
+    }
+  }
+  T* get() const
+  {
+    return resource_;
+  }
+  T* operator->() const
+  {
+    return resource_;
+  }
+  T* release()
+  {
+    T* ret = resource_;
+    resource_ = nullptr;
+    return ret;
+  }
+ private:
+  T* resource_;
+};
+
+typedef AutoFree<RSA, RSA_new, RSA_free> RSAWrap;
+typedef AutoFree<BIGNUM, BN_new, BN_free> BIGNUMWrap;
+END_NAMESPACE();
+
 std::string
 xgetpass(const std::string& prompt)
 {
   const int fd = STDIN_FILENO;
-  std::cout << prompt << ": " << std::flush;
+  std::cerr << prompt << ": " << std::flush;
   std::string line;
   if (!isatty(fd)) {
     getline(std::cin, line);
@@ -85,7 +183,7 @@ xgetpass(const std::string& prompt)
       throw std::runtime_error(std::string("tcsetattr(stdin, TCSAFLUSH, old): ") + strerror(errno));
     }
   }
-  std::cout << std::endl;
+  std::cerr << std::endl;
   return line;
 }
 
@@ -192,6 +290,16 @@ bn2string(const BIGNUM* bn)
   return std::string(buf.begin(), buf.end());
 }
 
+BIGNUM*
+string2bn(const std::string& s)
+{
+  BIGNUMWrap ret;
+  if (!BN_bin2bn(reinterpret_cast<const unsigned char*>(s.data()), s.size(), ret.get())) {
+    throw std::runtime_error("Broken BIGNUM sent to BN_bin2bn.");
+  }
+  return ret.release();
+}
+
 std::string
 xctime()
 {
@@ -267,16 +375,20 @@ generate_software_key(int bits)
     throw std::runtime_error("OpenSSL PRNG wants more entropy");
   }
 
-  RSA *rsa = RSA_new();
-  BIGNUM *f4 = BN_new();
-  BN_set_word(f4, RSA_F4);
-  if (!RSA_generate_key_ex(rsa, bits, f4, NULL)) {
+  RSAWrap rsa;
+  BIGNUMWrap f4;
+  BN_set_word(f4.get(), RSA_F4);
+  if (!RSA_generate_key_ex(rsa.get(), bits, f4.get(), NULL)) {
     throw std::runtime_error("RSA_generate_key_ex failed");
   }
   SoftwareKey swkey;
-  swkey.modulus = bn2string(rsa->n);
-  swkey.key = bn2string(rsa->p);
-  swkey.exponent = bn2string(rsa->e);
+  BIGNUM *m, *e;
+  RSA_get0_key(rsa.get(), const_cast<const BIGNUM**>(&m), const_cast<const BIGNUM**>(&e), NULL);
+  swkey.modulus = bn2string(m);
+  swkey.exponent = bn2string(e);
+  BIGNUM *p;
+  RSA_get0_factors(rsa.get(), const_cast<const BIGNUM**>(&p), NULL);
+  swkey.key = bn2string(p);
   return swkey;
 }
 
@@ -308,7 +420,7 @@ wrap_key(const std::string* srk_pin, const std::string* key_pin,
   // Load SRK public key.
   {
     UINT32 pubKeySize;
-    BYTE *pubKey;
+    BYTE *pubKey = nullptr;
     TSCALL(Tspi_Key_GetPubKey, stuff.srk(), &pubKeySize, &pubKey);
     Tspi_Context_FreeMemory(stuff.ctx(), pubKey);
   }
@@ -351,9 +463,12 @@ generate_key(const std::string* srk_pin, const std::string* key_pin, int bits) {
   TPMStuff stuff{srk_pin};
 
   { // Get some random data and seed the TPM with it.
-    const std::string entropy = xrandom(num_random_bytes);
-    TSCALL(Tspi_TPM_StirRandom, stuff.tpm(),
-           entropy.size(), (BYTE*)entropy.data());
+    const int chunk = 32;
+    for (int left =  num_random_bytes; left > 0; left -= chunk) {
+      const std::string entropy = xrandom(chunk);
+      TSCALL(Tspi_TPM_StirRandom, stuff.tpm(),
+             entropy.size(), (BYTE*)entropy.data());
+    }
   }
 
   // === Set up key object ===
@@ -682,6 +797,40 @@ sign(const Key& key, const std::string& data,
   TSCALL(Tspi_Hash_Sign, hash, sign, &sig_size, &sig_blob);
   return std::string{sig_blob, sig_blob+sig_size};
 }
+
+std::string
+public_decrypt(const Key& key, const std::string& sig)
+{
+  // Load key.
+  RSAWrap rsa;
+  if (RSA_set0_key(rsa.get(), string2bn(key.modulus), string2bn(key.exponent),
+                   NULL)) {
+    throw std::runtime_error("RSA_set0_key failed");
+  }
+
+  // Decrypt signature.
+  std::vector<unsigned char> d(RSA_size(rsa.get()));
+  const int len = RSA_public_decrypt(
+      sig.size(),
+      reinterpret_cast<const unsigned char*>(sig.data()),
+      &d[0],
+      rsa.get(),
+      RSA_PKCS1_PADDING);
+  if (len < 0) {
+    throw std::runtime_error(xsprintf("RSA_public_decrypt failed: %s", ERR_error_string(ERR_get_error(), NULL)));
+  }
+  return std::string{&d[0], &d[len]};
+}
+
+bool
+verify(const Key& key, const std::string& data, const std::string& sig)
+{
+  // TODO: Make this comparison constant time.
+  if (data != public_decrypt(key, sig)) {
+    return false;
+  }
+  return true;
+}
 END_NAMESPACE(stpm);
 /* ---- Emacs Variables ----
  * Local Variables:
